@@ -5,8 +5,9 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.3
+# Version   : 0.3.6
 
+from operator import contains
 import  numpy                   as      np
 import  plotly.graph_objects    as      go
 import  easygui                 as      gui
@@ -1294,107 +1295,201 @@ class AFM:
             )
         fig.show()
 
-    def adhesion_map(self,
+    def force_map(self,
         x_cells = 10,
         y_cells = 10,
         x_scale = 10,
         y_scale = 10,
-        return_fig = False
+        return_fig = False,
+        auto_cell = True,
+        plot = 'adhesion',
+        topography_3d = False
     ):
 
-        x_pos = np.zeros(x_cells * y_cells)
-        y_pos = np.zeros(x_cells * y_cells)
-        adhesion = np.zeros(x_cells * y_cells)
-        modulus = np.zeros(x_cells * y_cells)
-        depth = np.zeros(x_cells * y_cells)
+        plot_types = ['adhesion', 'modulus', 'topography', 'indent', 'max_force', 'snap_in']
+        member = plot in plot_types
 
-        x_counter = 1
-        y_counter = 1
-        loop_counter = 0
-        updown = 1
+        if member == False:
+            print('Incorrect plot type specified. Available plots are adhesion, modulus, topography, indent, max_force and snap_in.')
 
-        j = 0
+        else:
 
-        while j < x_cells * y_cells:
+            if auto_cell == True:
+                x_cells = int(np.sqrt(self.approach.shape[1]))
+                y_cells = x_cells
 
-            x_pos[j] = x_counter
-            y_pos[j] = y_counter
-            adhesion[j] = self.adhesion[j]
-            modulus[j] = self.modulus[j]
-            depth[j]= next(n for n in self.z_approach[:,j] if n == n)
+            x_pos = np.zeros(x_cells * y_cells)
+            y_pos = np.zeros(x_cells * y_cells)
+            adhesion = np.zeros(x_cells * y_cells)
+            modulus = np.zeros(x_cells * y_cells)
+            height = np.zeros(x_cells * y_cells)
+            depth = np.zeros(x_cells * y_cells)
+            force = np.zeros(x_cells * y_cells)
+            snap = np.zeros(x_cells * y_cells)
 
-            loop_counter += 1
+            x_counter = 1
+            y_counter = 1
+            loop_counter = 0
+            updown = 1
 
-            if loop_counter == x_cells:
+            j = 0
 
-                y_counter += 1
-                j += 1
-
-                if j == x_cells * y_cells:
-                    break
+            while j < x_cells * y_cells:
 
                 x_pos[j] = x_counter
                 y_pos[j] = y_counter
-                adhesion[j] = self.adhesion[j]
+
+                if plot == 'adhesion':
+                    adhesion[j] = self.adhesion[j]
+
+                if plot == 'modulus':
+                    modulus[j] = self.modulus[j]
+
+                if plot == 'indent':
+                    depth[j] = self.indent_depth[j]
+
+                if plot == 'max_force':
+                    force[j] = self.max_force[j]
+
+                if plot == 'snap_in':
+                    snap[j] = self.snap_in[j]
+
+                height[j] = self.contact_point[j]
+
+                loop_counter += 1
+
+                if loop_counter == x_cells:
+
+                    y_counter += 1
+                    j += 1
+
+                    if j == x_cells * y_cells:
+                        break
+
+                    x_pos[j] = x_counter
+                    y_pos[j] = y_counter
+                    
+                    if plot == 'adhesion':
+                        adhesion[j] = self.adhesion[j]
+
+                    if plot == 'modulus':
+                        modulus[j] = self.modulus[j]
+
+                    if plot == 'indent':
+                        depth[j] = self.indent_depth[j]
+
+                    if plot == 'max_force':
+                        force[j] = self.max_force[j]
+
+                    if plot == 'snap_in':
+                        snap[j] = self.snap_in[j]
+
+                    height[j] = self.contact_point[j]
+
+                    if updown == 1:
+                        updown = 2
+                        loop_counter = 1
+                    elif updown == 2:
+                        updown = 1
+                        loop_counter = 1
 
                 if updown == 1:
-                    updown = 2
-                    loop_counter = 1
+                    x_counter += 1
                 elif updown == 2:
-                    updown = 1
-                    loop_counter = 1
+                    x_counter -= 1
 
-            if updown == 1:
-                x_counter += 1
-            elif updown == 2:
-                x_counter -= 1
+                j += 1
 
-            j += 1
+            x_grid = x_pos
+            y_grid = y_pos
 
-        x_pos = x_pos * x_scale/x_cells
-        y_pos = y_pos * y_scale/y_cells
-        depth = depth - depth[0]
+            x_pos = x_pos * x_scale/x_cells
+            y_pos = y_pos * y_scale/y_cells
 
-        fig = go.Figure(data=go.Heatmap(
-                x = x_pos,
-                y = y_pos,
-                z = adhesion,
-                type = 'heatmap',
-                colorscale = 'YlOrBr',
-                colorbar=dict(title='Adhesion (nN)')
-                ))
+            if plot == 'adhesion':
+                plt_title = 'Adhesion (nN)'
+                plt_data = adhesion
+            elif plot == 'modulus':
+                plt_title = 'Young''s modulus (MPa)'
+                plt_data = modulus
+            elif plot == 'topography':
+                plt_title = 'Height (nm)'
+                plt_data = height
+            elif plot == 'indent':
+                plt_title = 'Indentation depth (nm)'
+                plt_data = depth
+            elif plot == 'max_force':
+                plt_title = 'Max. indentation force (nN)'
+                plt_data = force
+            elif plot == 'snap_in':
+                plt_title = 'Snap in force (nN)'
+                plt_data = snap
 
-        fig.update_layout(yaxis = dict(scaleanchor = 'x'))
-        fig.update_layout(plot_bgcolor='rgba(0,0,0,0)')
+            if topography_3d == False:
 
-        axis_template = dict(
-                    showgrid = False, zeroline = False,
-                    linecolor = 'white', showticklabels = True,
-                    ticks = '' )
+                fig = go.Figure(data=go.Heatmap(
+                        x = x_pos,
+                        y = y_pos,
+                        z = plt_data,
+                        type = 'heatmap',
+                        colorscale = 'YlOrBr',
+                        colorbar=dict(title=plt_title)
+                        ))
 
-        fig.update_layout(margin = dict(t=200,r=200,b=200,l=200),
-            xaxis = axis_template,
-            yaxis = axis_template,
-            showlegend = False,
-            width = 700, height = 700,
-            autosize = False,
-            xaxis_title="X Position (um)",
-            yaxis_title="Y Position (um)")
+                fig.update_layout(yaxis = dict(scaleanchor = 'x'))
+                fig.update_layout(plot_bgcolor='rgba(0,0,0,0)')
 
-        fig.update_layout(
-            xaxis = dict(
-                tickmode = 'linear',
-                tick0 = 0,
-                dtick = 1
-            ),
-                yaxis = dict(
-                tickmode = 'linear',
-                tick0 = 0,
-                dtick = 1
-            )
-        )
+                axis_template = dict(
+                            showgrid = False, zeroline = False,
+                            linecolor = 'white', showticklabels = True,
+                            ticks = '' )
 
-        fig.show()
+                fig.update_layout(margin = dict(t=200,r=200,b=200,l=200),
+                    xaxis = axis_template,
+                    yaxis = axis_template,
+                    showlegend = False,
+                    width = 700, height = 700,
+                    autosize = False,
+                    xaxis_title="X Position (um)",
+                    yaxis_title="Y Position (um)")
 
-        if return_fig == True:
-            return fig
\ No newline at end of file
+                fig.update_layout(
+                    xaxis = dict(
+                        tickmode = 'linear',
+                        tick0 = 0,
+                        dtick = 1
+                    ),
+                        yaxis = dict(
+                        tickmode = 'linear',
+                        tick0 = 0,
+                        dtick = 1
+                    )
+                )
+
+            elif topography_3d == True:
+
+                x3 = np.zeros([x_cells, y_cells])
+                y3 = np.zeros([x_cells, y_cells])
+                z3 = np.zeros([x_cells, y_cells])
+                d3 = np.zeros([x_cells, y_cells])
+
+                for i in range(0,len(plt_data)):
+
+                    x3[int(x_grid[i])-1 , int(y_grid[i])-1] = x_pos[i]
+                    y3[int(x_grid[i])-1 , int(y_grid[i])-1] = y_pos[i]
+                    z3[int(x_grid[i])-1 , int(y_grid[i])-1] = height[i]
+                    d3[int(x_grid[i])-1 , int(y_grid[i])-1] = plt_data[i]
+
+                fig = go.Figure(go.Surface(x=x3, y=y3, z=z3, surfacecolor=d3,
+                                colorbar=dict(title=plt_title)))
+
+                fig.update_layout(scene = dict(
+                                xaxis_title='X Position (um)',
+                                yaxis_title='Y Position (um)',
+                                zaxis_title='Z Position (um)'),
+                                margin=dict(r=20, b=10, l=10, t=10))
+
+            fig.show()
+
+            if return_fig == True:
+                return fig
\ No newline at end of file
